/*
 * Created on October 26, 2005, 4:58 PM
 */

/*
 * Copyright ??? 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *
 */

package com.sunlabs.archsim.fleet.ships.accumulator;


import java.util.List;
import java.util.Iterator;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;


import com.sunlabs.archsim.fleet.ships.testing.DataSink;

import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 *
 * @author mh193489
 */
public class TestAccumulator extends TestCase {
    
    private static String fileName= "src/com/sunlabs/archsim/fleet/ships/accumulator/TestAccumulatorScript.xml";
    
    private static Director director;
    
    private static final String DATA_SOURCE= "DataSource";
    private static final String DATA_SINK= "DataSink";

    /* reference to the DataSink SHIP for result inspection */
    private DataSink dataSink;

    protected void setUp() {
        
        // for sum test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(2));
        PayloadSource.add(DATA_SOURCE, new Data(3));
        PayloadSource.add(DATA_SOURCE, new Data(5));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // for resest test
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // for overflow test
        PayloadSource.add(DATA_SOURCE, new Data(AccumulatorDefaults.OVERFLOW));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // for reset after overflow test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // for underflow test
        PayloadSource.add(DATA_SOURCE, new Data(AccumulatorDefaults.UNDERFLOW));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // for reset after underflow test
        PayloadSource.add(DATA_SOURCE, new Data(1));
        PayloadSource.add(DATA_SOURCE, new Data(false));
        
        // initialize the tests
        director= Director.getInstance();
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(fileName);
        
        Runner r= new Runner();
        r.perform(ops);
        
        /* get the reference to current DataSink SHIP instance */
        dataSink= (DataSink)Entities.find(DATA_SINK);
  
        
    } //setUp
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        // director.reset();
        director.masterClear();
    } // tearDown
    
    public static Test suite() {
        return new TestSuite(TestAccumulator.class);
    }
    
    public static void main(String[] args) {
        // run the tests
        junit.textui.TestRunner.run(TestAccumulator.class);
        
        
        
    } // main
    
    public void testTestSetup() {
        // data sink
        assertNotNull(dataSink);
        assertNotNull(dataSink.receivedData);
        for (Iterator<Data> i= dataSink.receivedData.iterator(); i.hasNext();) {
            // check if data is not null
            Data data= i.next();
            assertNotNull(data);
            // check if data value is not null, in case data is valid
            if (data.valid) {
                assertNotNull(data.value);
            }
        }
        
    } 
    
    
    public void testSum() {
        assertEquals( ((Integer)dataSink.receivedData.get(0).value).intValue(), 11);
    }
    
    public void testReset() {
        // check if the second data output is 0, which was produced after reset
        assertTrue(dataSink.receivedData.get(1).valid);
        assertEquals( ((Integer)dataSink.receivedData.get(1).value).intValue(), 0);
    }
    
    public void testOverflow() {
        assertFalse(dataSink.receivedData.get(2).valid);
        assertEquals(dataSink.receivedData.get(2).value, AccumulatorDefaults.PLUS_INFINITY);
    }
    
    public void testResetAfterOverflow() {
        assertTrue(dataSink.receivedData.get(3).valid);
        assertEquals(dataSink.receivedData.get(3).value, 1);
    }
    
    public void testUnderflow() {
        assertFalse(dataSink.receivedData.get(4).valid);
        assertEquals(dataSink.receivedData.get(4).value, AccumulatorDefaults.MINUS_INFINITY);
    }
    
    public void testResetAfterUnderflow() {
        assertTrue(dataSink.receivedData.get(5).valid);
        assertEquals(dataSink.receivedData.get(5).value, 1);
    }
    
}
